(function(window, undefined){
	// Plugin itself:
	// Note - localtion could be ~/.local/share/onlyoffice/desktopeditors/sdkjs-plugins/... )

	var verbose = false;

	var isDefined = function(v) {
		// check undefined in strict mode
		return typeof v !== 'undefined';
	}

	var getDocTitle = function (infoObj) {
		// infoObj === window.Asc.plugin.info
		// getter for document title
		// reason: "title" by docs but "documentTitle" was in real life)
		// Ref: https://api.onlyoffice.com/plugin/info
		try {
			return infoObj.documentTitle
		} catch (e) {
			return infoObj.title
		}
	}

	var httpclient = function (url, method, callback, postData) {
		// References: https://ru.wikipedia.org/wiki/XMLHttpRequest
		method = method ? method : 'GET';
		callback = callback ? callback : function(respText) {};
		var httpRequest;
		if (window.XMLHttpRequest) { // Mozilla, Safari, ...
			httpRequest = new XMLHttpRequest();
		} else if (window.ActiveXObject) { // IE
			try {
				httpRequest = new ActiveXObject("Msxml2.XMLHTTP");
			} catch (e) {
				try {
					httpRequest = new ActiveXObject("Microsoft.XMLHTTP");
				} catch (e) {
				}
			}
		}

		httpRequest.onreadystatechange = function () {
			if (httpRequest.readyState === 4) {
				callback(httpRequest.status === 200 ? httpRequest.responseText : null);
			}
		};
		httpRequest.open(method, url, true);
		httpRequest.setRequestHeader("Content-Type", "application/json; charset=UTF-8")
		if (method === 'POST' && postData) {
			payload = JSON.stringify(postData);
			httpRequest.send(payload);
		} else {
			httpRequest.send();
		}
	}

	var bridge = {};

	bridge.endpoint = "http://127.0.0.1:8987";

	bridge.createKey = function (docTitle, callback) {
		var url = bridge.endpoint + "/session";
		httpclient(url, 'POST',  function (value) {
			if (value) {
				try {
					verbose && console.log("bridge.createKey > got: ", value);
					callback(JSON.parse(value));
				} catch (e) {
					console.error('bridge.createKey - something wrong', e);
				}
			} else {
				verbose && console.debug('bridge.createKey > nothing');
			}
			// //if you need more information check this page https://api.onlyoffice.com/plugin/executecommand
			// t.executeCommand('close', command);
		}, {"s_id": docTitle});
	};

	bridge.readCommand = function (s_id, callback) {
		var url = bridge.endpoint + "/read/" + s_id;
		httpclient(url, 'GET',  function (value) {
			if (value) {
				try {
					verbose && console.log("bridge.readCommand > got: ", value);
					callback(JSON.parse(value));
				} catch (e) {
					console.error('bridge.readCommand', e);
				}
			} else {
				verbose && console.debug('bridge.readCommand > nothing');
			}
		});		
	};

	bridge.sendAck = function (s_id, c_id, success, msg, callback) {
		var url = bridge.endpoint + "/ack/" + s_id + "/" + c_id;
		httpclient(url, 'POST',  function (value) {
			if (value) {
				try {
					console.log("bridge.sendAck > got: ", value);
					callback(JSON.parse(value));
				} catch (e) {
					console.error('bridgebridge.sendAck', e);
				}
			} else {
				verbose && console.debug('bridge.sendAck > nothing');
			}
			// //if you need more information check this page https://api.onlyoffice.com/plugin/executecommand
			// t.executeCommand('close', command);
		}, {"success": success, "message": msg})
	};

	
	window.Asc.plugin.init = function()
    {
    	// Define app namespace:
    	window.Asc.plugin.robin = {
    		session_key: null, 
    		doc_name: null
    	};

    	this.docIsReady = false;


    	var self = this;


    	console.log("VERSION", window.Asc.plugin.version);

		// // // plugin methods - executeMethod, SetMarcos: https://api.onlyoffice.com/plugin/executemethod/common/setmacros
		// // // More: https://api.onlyoffice.com/plugin/executemethod/common/setmacros
		// // // setInterval: https://api.onlyoffice.com/plugin/macrosamples/recalculatevalues
		// // window.Asc.plugin.executeMethod ("SetMacros", [JSON.stringify(Content)], function () {
		// //     window.Asc.plugin.executeCommand ("close", "");
		// // });

		// // MORE: - macros plugin:
		// 	// /opt/onlyoffice/desktopeditors/editors/sdkjs-plugins/{E6978D28-0441-4BD7-8346-82FAD68BCA3B}/scripts/code.js
		// // It is possible to fetch internet data with macros: https://www.onlyoffice.com/blog/2023/07/retrieve-website-details-with-onlyoffice-macro 
		// ================================================

		var dataMessage = {
			frameEditorId : "iframeEditor",

			// The same guid as in config .json (recipient - "self")
			guid : "asc.{F6406795-E291-4065-971E-18AA2DF47607}",
			type : "onExternalPluginMessage",
			data : {
				type: "pollConnector",
				payload: ""
			}
		};

		var cb = function (){
			window.postMessage(JSON.stringify(dataMessage), "*");
			verbose && console.log("Document Worker > message sent");
		};

		console.log("RpaRobin - Init");

		setTimeout(function(){
			document.robin_timer_handle = setInterval(cb, 1000)
		}, 0);
		console.log("RpaRobin - Timer installed");

		// ================================================

    };

    window.Asc.plugin.event_onDocumentContentReady = function () {
    }


    window.Asc.plugin.event_onDocumentReady = function () {
    }

    // onInfo 
    window.Asc.plugin.event_onInfo = function (data) {
    }

    // Event handler for external commands:
	window.Asc.plugin.onExternalPluginMessage = function(data)
	{
		// console.log("Plugin: got command", data);
		verbose && console.log("window.Asc.plugin.info ---> ", window.Asc.plugin.info );

		verbose && console.log("window.Asc.plugin.info.documentTitle ", window.Asc.plugin.info.documentTitle);
		verbose && console.log("window.Asc.plugin.info.editorType ", window.Asc.plugin.info.editorType);

		switch (data.type)
		{
			case "pollConnector":
			{

				// Update session key because bridge can be restarted
				var docTitle = getDocTitle(window.Asc.plugin.info);
				if (docTitle) {
					bridge.createKey(docTitle, function (resp) {
						if (resp) {
							console.log("I got KEY", resp);
							window.Asc.plugin.robin.session_key = resp["s_key"];
						}
					});
				}

				// this.executeCommand("close", "");
				if (window.Asc.plugin.robin.session_key) {
					bridge.readCommand(window.Asc.plugin.robin.session_key, function (data) {
						verbose && console.log("plugin > bridge.readCommand > got: ", data);

						var c_sample = ['Api.GetActiveSheet().GetRange("B3").SetFillColor(Api.CreateColorFromRGB(0, 0, 250));'];
						//  STUB!!!:
						// data.cmd = c_sample;

						if (data && data.cmd.length > 0) {
							verbose && console.log("command: ", data.cmd);
							var sScript = data.cmd.join(';');
							// this.executeCustomCommand(cText);

							// Hardcoded for a while, move this to protocol attribute:
					        window.Asc.plugin.info.recalculate = true;

					        var s_id = window.Asc.plugin.robin.session_key;
					        try {
					        	console.log("bridge < command");
	        					window.Asc.plugin.executeCommand("command", sScript);
	        					// All is good:
	        					bridge.sendAck(s_id, data.c_id, true, "", function(){
	        						console.log("bridge > ACK");
	        					});
					        } catch (e) {
	        					// All is good:
	        					bridge.sendAck(s_id, data.c_id, false, e.toString(), function(){
	        						console.log("bridge > NACK", e);
	        					});
					        }
						}
					});
				}
				break;
			}

		}
	}


	window.Asc.plugin.button = function(id)
    {
     //    this.executeCommand("close", "");
    };

})(window, undefined);
